package main;

import java.awt.*;
import java.awt.event.*;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.*;

import main.notepage.Notepage;
/**
 * Governs all the GUI components of the main window in our application.
 * 
 * @author Wervyn
 * @version 2.0
 */
@SuppressWarnings("serial")
public class MainWindow extends JFrame {
	
	/** Reference to the sidebar panel of the application */
	private JPanel sidebar;
	
	/** Reference to the main content panel of the application */
	private JPanel mainContent;
	
	/** Reference to the statusbar panel of the application */
	private JPanel statusbar;
	
	/** Reference to the status label (needs class-scope variable for access) */
	private JLabel statuslabel;
	
	/** Reference to the menubar for the application */
	private JMenuBar menubar;
	
	/** Reference to the File menu on the menu bar */
	private JMenu fileMenu;
	
	/** Reference to the Display menu on the menu bar */
	private JMenu displayMenu;
	
	/** Reference to the Page Sorter toggle option in the menus (needs access) */
	private JMenuItem pageSorterLabel;
	
	/** Reference to the Color selector for Notepage tools. */
	private JComboBox colorTool;
	
	/** Reference to the Font selector for Notepage tools. */
	private JComboBox fontTool;
	
	/** Reference to the Size selector for Notepage tools.*/
	private JComboBox sizeTool;
	
	/** Reference to the Page Sorter frame for the application, child of the main window */
	private PageSorter pageSorter;
	
	/** Reference to the currently active Notepage */
	private Notepage currentPage;
	
	/** A list of all the note pages. */
	private Vector<Notepage> pages;
	
	/** A mapping of tag names to tag controls. */
	private HashMap<String, JCheckBox> tagBoxes;
	
	/** A mapping of color names to Color objects. */
	private HashMap<String, Color> colorMap;
	
	/** A mapping of font names to Font objects. */
	private HashMap<String, Font> fontMap;
	
	/** The currently active editing mode. */
	private int mode;
	
	/**
	 * Basic constructor, sets up all GUI relationships in the application.
	 */
	public MainWindow() {
		// Main component setup //
		super("Notebook Application");
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setSize(640,480);
		this.setMinimumSize(new Dimension(480,400)); // Prevents odd effects of the window becoming too small.
		Container content = this.getContentPane();
		content.setLayout(new BorderLayout());
		
		// Delegation of child component setup //
		this.setJMenuBar(initMenubar());
		content.add(initSidebar(), BorderLayout.WEST);
		content.add(initStatusbar(), BorderLayout.SOUTH);
		content.add(initMainContent(), BorderLayout.CENTER);
		initPageSorter();
		setMode(Notepage.PEN_MODE);
	}
	
	/**
	 * Start the application (i.e. display the main window).
	 */
	public void start() {
		this.setVisible(true);
	}
	
	/**
	 * Initialize the menubar of the application.
	 * 
	 * @return A reference to the initialized menubar.
	 */
	protected JMenuBar initMenubar() {
		menubar = new JMenuBar();
		
		// Setup File Menu //
		fileMenu = new JMenu("File");
		// 		New Page //
		JMenuItem item = new JMenuItem("New Page");
		item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				logEvent("New Page Created");
				newPage();
			}
		});
		fileMenu.add(item);
		//		Delete Page //
		item = new JMenuItem("Delete Page");
		item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				logEvent("Current Page Deleted");
				deletePage();
			}
		});
		//		Exit //
		fileMenu.add(item);
		item = new JMenuItem("Exit");
		item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.exit(0);  // More conceptually correct way to do this?
				// Should be fine for now.
			}
		});
		fileMenu.add(item);
		menubar.add(fileMenu);
		
		// Setup Display Menu //
		displayMenu = new JMenu("Display");
		// 		<Toggle> Page Sorter //
		pageSorterLabel = new JMenuItem("Show Page Sorter");
		pageSorterLabel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				togglePageSorter(); // The actual text of this label changes
				// based on program state, and since a CheckBoxMenuItem isn't
				// good enough for this project, we'll handle it ourselves.
			}
		});
		displayMenu.add(pageSorterLabel);
		menubar.add(displayMenu);
		
		return menubar;
	}
	
	/**
	 * Initialize the status bar at the bottom of the application window.
	 * 
	 * @return A reference to the initialized status bar.
	 */
	protected JPanel initStatusbar() {
		statusbar = new JPanel();
		statuslabel = new JLabel(" "); // Gives the label a spatial presence.
		statusbar.add(statuslabel);
		return statusbar;
	}
	
	/**
	 * Initialize the sidebar of controls.
	 * 
	 * @return A reference to the initialized sidebar
	 */
	protected JPanel initSidebar() {
		sidebar = new JPanel();
		sidebar.setLayout(new BoxLayout(sidebar, BoxLayout.PAGE_AXIS));
		JPanel tools, tags, nav;
		
		// Setup Tool Panel //
		tools = new JPanel();
		tools.setLayout(new BoxLayout(tools, BoxLayout.PAGE_AXIS));
		ButtonGroup rbuttons = new ButtonGroup(); // Tools have the property 
			// of a radio button group: only one can be active at a time.
		//		Text Tool //
		JRadioButton rbutton = new JRadioButton("Text");
		rbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				logEvent("Text Tool Selected");
				setMode(Notepage.TEXT_MODE);
			}
		});
		rbuttons.add(rbutton);
		tools.add(rbutton);
		//		Pen Tool //
		rbutton = new JRadioButton("Pen");
		rbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				logEvent("Pen Tool Selected");
				setMode(Notepage.PEN_MODE);
			}
		});
		rbutton.setSelected(true);
		rbuttons.add(rbutton);
		tools.add(rbutton);
		//		Color Selector //
		colorTool = new JComboBox();
		colorTool.setMaximumSize(new Dimension(100, 25));
		colorMap = new HashMap<String, Color>();
		colorTool.setName("Color");
		colorTool.addItem("Black");
		colorMap.put("Black", Color.black);
		colorTool.addItem("Blue");
		colorMap.put("Blue", Color.blue);
		colorTool.addItem("Red");
		colorMap.put("Red", Color.red);
		colorTool.addItem("Green");
		colorMap.put("Green", Color.green);
		colorTool.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				selectColor();
			}
		});
		colorTool.setSelectedIndex(0);
		tools.add(colorTool);
		//		Font Selector //
		fontTool = new JComboBox();
		fontTool.setMaximumSize(new Dimension(100, 25));
		fontMap = new HashMap<String, Font>();
		fontTool.setName("Font");
		fontTool.addItem("Arial");
		fontMap.put("Arial", new Font("Arial", Font.BOLD, 10));
		fontTool.addItem("Georgia");
		fontMap.put("Georgia", new Font("Georgia", Font.BOLD, 10));
		fontTool.addItem("Verdana");
		fontMap.put("Verdana", new Font("Verdana", Font.BOLD, 10));
		fontTool.addItem("Times New Roman");
		fontMap.put("Times New Roman", new Font("Times New Roman", Font.BOLD, 10));
		fontTool.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				selectFont();
			}
		});
		fontTool.setSelectedIndex(0);
		tools.add(fontTool);
		//		Size Selector //
		sizeTool = new JComboBox();
		sizeTool.setMaximumSize(new Dimension(100, 25));
		sizeTool.setName("Size");
		sizeTool.addItem("1");
		sizeTool.addItem("2");
		sizeTool.addItem("3");
		sizeTool.addItem("4");
		sizeTool.addItem("5");
		sizeTool.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				selectSize();
			}
		});
		sizeTool.setSelectedIndex(2);
		tools.add(sizeTool);
		// Setup Tag Panel //
		tags = new JPanel();
		tagBoxes = new HashMap<String, JCheckBox>();
		tags.setLayout(new BoxLayout(tags, BoxLayout.PAGE_AXIS));
		
		final String[] tbuttonNames = { "Appointment", "Action Item", "Personal", "In Progress" };
		// In order to expedite changes later, we'll use a loop here with variable input.
		// In the future this will probably be dynamically generated from user config, and
		// modifiable through the application.
		JCheckBox tbutton;
		for (int i = 0; i < tbuttonNames.length; i++) {
			final String temp = tbuttonNames[i]; // An unfortunate kludge so that the anonymous
				// class for the event handler knows what we're talking about.
			tbutton = new JCheckBox(temp);
			tagBoxes.put(temp, tbutton);
			tbutton.setAlignmentX(Component.LEFT_ALIGNMENT);
			tbutton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					logEvent(temp + " Tag Toggled");
					toggleTag(temp); 
				}
			});
			tags.add(tbutton);
		}

		// Setup Navigation Panel //
		nav = new JPanel(new FlowLayout());
		//		Previous Page //
		JButton button = new JButton("<< Prev");
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				logEvent("Previous Page Selected");
				prevPage();
			}
		});
		nav.add(button);
		//		Next Page //
		button = new JButton("Next >>");
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				logEvent("Next Page Selected");
				nextPage();
			}
		});
		nav.add(button);
		nav.setMaximumSize(new Dimension(200,50));
		
		// Compile all components //
		tools.setAlignmentX(Component.RIGHT_ALIGNMENT);
		sidebar.add(tools);
		sidebar.add(Box.createVerticalGlue());
		tags.setAlignmentX(Component.RIGHT_ALIGNMENT);
		sidebar.add(tags);
		sidebar.add(Box.createVerticalGlue());
		nav.setAlignmentX(Component.RIGHT_ALIGNMENT);
		sidebar.add(nav);
		sidebar.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		return sidebar;
	}
	
	/**
	 * Initialize the main content panel.
	 * 
	 * @return A reference to the content panel.
	 */
	protected JPanel initMainContent() {
		pages = new Vector<Notepage>();
		currentPage = new Notepage();
		mainContent = new JPanel();
		mainContent.setLayout(new BorderLayout());
		mainContent.add(currentPage);
		pages.add(currentPage);
		return mainContent;
	}
	
	/**
	 * Initialize the page sorter frame.  There is no return because
	 * we don't immediately do anything with it, but style may dictate that
	 * there should be anyway.
	 */
	protected void initPageSorter() {
		pageSorter = new PageSorter(this);
		pageSorter.addWindowListener(new WindowListener() {
			public void windowActivated(WindowEvent e) {}
			public void windowClosed(WindowEvent e) { hidePageSorter(); }
			public void windowClosing(WindowEvent e) { hidePageSorter(); }
			public void windowDeactivated(WindowEvent e) {}
			public void windowDeiconified(WindowEvent e) {}
			public void windowIconified(WindowEvent e) {}
			public void windowOpened(WindowEvent e) { showPageSorter(); }
		});
		// This anonymous WindowListener is primarily so that we can make changes
		// to other parts of the app based on the status of the PageSorter window.
	}
	
	/**
	 * Toggle the visibility of the PageSorter Window.
	 */
	protected void togglePageSorter() {
		if (pageSorter.isVisible()) hidePageSorter();
		else showPageSorter();
	}
	
	/**
	 * Make the PageSorter window visible.
	 */
	protected void showPageSorter() {
		pageSorter.setVisible(true);
		pageSorterLabel.setText("Hide Page Sorter");
		logEvent("Page Sorter Shown");
	}
	
	/**
	 * Make the PageSorter window invisible.
	 */
	protected void hidePageSorter() {
		pageSorter.setVisible(false);
		pageSorterLabel.setText("Show Page Sorter");
		logEvent("Page Sorter Hidden");
	}
	
	/**
	 * Set the drawing mode of the application (currently Text or Pen)
	 * 
	 * @param mode The new value for mode.
	 */
	protected void setMode(int mode) {
		this.mode = mode;
		currentPage.setMode(mode);
	}
	
	/**
	 * Get the drawing mode of the application
	 * 
	 * @return The current drawing mode.
	 */
	protected int getMode() {
		return this.mode;
	}

	/**
	 * Apply the currently selected color to the current Notepage.
	 */
	protected void selectColor() {
		if (currentPage != null) { // For safety, normally not an issue but can happen during init.
			currentPage.setColor(colorMap.get(colorTool.getSelectedItem().toString()));
		}
	}
	
	/**
	 * Apply the currently selected font (and scaled size) to the current Notepage.
	 */
	protected void selectFont() {
		if (currentPage != null) {
			int size = new Integer(sizeTool.getSelectedItem().toString()).intValue();
			Font font = fontMap.get(fontTool.getSelectedItem().toString());
			Font newfont = new Font(font.getFontName(), font.getStyle(), size*2+8);
			currentPage.setFont(newfont);
		}
	}

	/**
	 * Apply the currently selected size (and to the font by proxy) to the current Notepage.
	 */
	protected void selectSize() {
		if (currentPage != null) {
			int size = new Integer(sizeTool.getSelectedItem().toString()).intValue();
			currentPage.setPenWidth(size);
			selectFont();
		}
	}
	
	/**
	 * Create a new, blank Notepage.
	 */
	protected void newPage() {
		currentPage = new Notepage();
		pages.addElement(currentPage);
		reloadPage();
	}
	
	/**
	 * Delete the current Notepage (switches to next page in the list or a blank page
	 * if the last page was deleted).
	 */
	protected void deletePage() {
		int index = pages.indexOf(currentPage);
		pages.removeElement(currentPage);
		if (pages.size() == 0) {
			newPage();
		} else {
			if (index >= pages.size()) {
				index = 0;
			}
			currentPage = pages.elementAt(index);
			reloadPage();
		}
	}
	
	/**
	 * Go to the next Notepage in the list (wraps around).
	 */
	protected void nextPage() {
		int index = pages.indexOf(currentPage);
		index = (index + 1) % pages.size();
		logEvent("Page " + (index+1) + " of " + pages.size() + " selected");
		currentPage = pages.elementAt(index);
		reloadPage();
	}
	
	/**
	 * Go to the previous Notepage in the list (wraps around).
	 */
	protected void prevPage() {
		int index = pages.indexOf(currentPage);
		index = (index + pages.size() - 1) % pages.size(); // Thank you Java for implementing modulo incorrectly...
		logEvent("Page " + (index+1) + " of " + pages.size() + " selected");
		currentPage = pages.elementAt(index);
		reloadPage();
	}
	
	/**
	 * Reinitialize and repaint the current Notepage, applying all current settings
	 * and retrieving all tags.
	 */
	protected void reloadPage() {
		mainContent.removeAll();
		mainContent.add(currentPage);
		currentPage.setMode(getMode());
		selectColor();
		selectFont();
		selectSize();
		Iterator<JCheckBox> tbuttons = tagBoxes.values().iterator();
		while (tbuttons.hasNext()) {
			tbuttons.next().setSelected(false);
		}
		Iterator<String> tags = currentPage.getTags().iterator();
		while (tags.hasNext()) {
			JCheckBox tbutton = tagBoxes.get(tags.next());
			tbutton.setSelected(true);
		}
	}
	
	/**
	 * Post the given text to the statusbar.
	 * 
	 * @param text The status or error message to be displayed.
	 */
	protected void logEvent(String text) {
		statuslabel.setText(text);
		// TODO: Write actual logging functionality?
	}
	
	/**
	 * Toggles the state of the given tag in the data model.
	 * 
	 * @param tag The name of the tag.
	 */
	protected void toggleTag(String tag) {
		JCheckBox tbutton = tagBoxes.get(tag);
		if (tbutton.isSelected()) {
			currentPage.addTag(tag);
		} else {
			currentPage.removeTag(tag);
		}
	}
}
