package main.notepage;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.plaf.ComponentUI;


/**
 * A basic implementation of Notepage UI, with support for drawing and typing.
 * 
 * @author Wervyn
 * @version 1.0
 */
public class BasicNotepageUI extends NotepageUI implements MouseListener, MouseMotionListener, KeyListener {

	/** The mouse stroke currently being drawn. */
	private NotepageStroke currentStroke = null;
	
	/** The text field currently being edited. */
	private NotepageText currentText = null;
	
	/** A flag which indicates whether the mouse is considered to be pressed or not. */
	private boolean mousePressed;
	
	/*
	 * (non-Javadoc)
	 * @see javax.swing.plaf.ComponentUI#paint(java.awt.Graphics, javax.swing.JComponent)
	 */
	public void paint(Graphics g, JComponent c) {

		// Draw the background first.
		paintBackground(g, c.getWidth(), c.getHeight(), 20);
		
		// Draw all existing elements on the Notepage.
		Vector<NotepagePrimative> objects = ((Notepage) c).getObjects();
		Iterator<NotepagePrimative> object = objects.iterator();
		while (object.hasNext()) {
			NotepagePrimative n = object.next();
			if (n.getType().equals(NotepageStroke.CLASS_ID))
				paintStroke(g, (NotepageStroke) n);
			if (n.getType().equals(NotepageText.CLASS_ID))
				paintText(g, (NotepageText) n, c.getWidth(), c.getHeight());
		}
		
		// Draw the currently active object, depending on the mode of the Notepage.
		if (((Notepage) c).getMode() == Notepage.PEN_MODE) {
			if (currentStroke != null && currentStroke.size() > 0) {
				paintStroke(g, currentStroke);
			}
		} else if (((Notepage) c).getMode() == Notepage.TEXT_MODE) {
			if (currentText != null) {
				Point pos = paintText(g, currentText, c.getWidth(), c.getHeight());
				g.setColor(Color.black);
				g.fillRect(pos.x, pos.y, 10, 3); // Draw a cursor so that we can see what we're editing.
			}
		}
	}
	
	/**
	 * Draw a background that looks like a piece of ruled notebook paper.
	 * 
	 * @param g The graphics object for use in drawing.
	 * @param width The width of area being drawn.
	 * @param height The height of the area being drawn.
	 * @param scale The scale of the ruled lines (distance in pixels between horizontal lines).
	 */
	protected void paintBackground(Graphics g, int width, int height, int scale) {
		g.setColor(Color.white);
		g.fillRect(0, 0, width, height); // Background
		g.setColor(Color.blue);
		for (int i = 0; i < height/scale; i++) {
			g.drawLine(0, (i+1)*scale, width, (i+1)*scale); // Horizontal lines
		}
		g.setColor(Color.red);
		g.drawLine(scale*5/2, 0, scale*5/2, height); // Doubled left margin.
		g.drawLine(scale*11/4, 0, scale*11/4, height);
	}
	
	/**
	 * Draw a NotepageStroke object.
	 * 
	 * @param g The graphics object for use in drawing.
	 * @param s The stroke to be drawn.
	 */
	protected void paintStroke(Graphics g, NotepageStroke s) {
		g.setColor(s.getPencolor()); 					// Color
		Graphics2D g2 = (Graphics2D) g;
		g2.setStroke(new BasicStroke(s.getPenwidth())); // Width
		Iterator<Point> points = s.iterator();
		if (points.hasNext()) {							// Stroke
			Point src, dest = points.next();
			while (points.hasNext()) {
				src = dest;
				dest = points.next();
				g.drawLine(src.x, src.y, dest.x, dest.y);
			}			
		}
	}
	
	/**
	 * Draw a NotepageText object.
	 * 
	 * @param g The graphics object for use in drawing.
	 * @param t The text to be drawn.
	 * @param width The width of the drawing area (for use in wrapping)
	 * @param height The height of the drawing area (for use in clipping)
	 * @return A location for the cursor, after drawing the text
	 */
	protected Point paintText(Graphics g, NotepageText t, int width, int height) {
		if (t.getLocation().x > width) return t.getLocation(); // Don't bother drawing anything if the text is off screen.
		String text = t.getText();
		String buffer = "";
		Font font = t.getFont();
		g.setFont(font);
		g.setColor(t.getColor());
		FontMetrics metrics = g.getFontMetrics();
		Point start = t.getLocation();
		int advance = 0;
		String[] words = text.split(" ");
		for (int i = 0; i < words.length; i++) { // Iterate over each word in the text, adding to a buffer.
			if (metrics.stringWidth(buffer + words[i]) > width - start.x) { // If the buffer would exceed the line length,
				g.drawString(buffer, start.x, start.y + advance);			// draw it and advance the line
				buffer = "";
				if (i != 0) {												// Edge case: The first word is too long for the line.
					advance += metrics.getHeight();							// So we only want to advance if we've already drawn something.
				}
				if (advance > height - start.y) break; // We can stop drawing if we reach the bottom of the page.
			}
			if (buffer.equals("") && metrics.stringWidth(words[i]) > width - start.x) { // Special case: the current word is longer than the line.
				int breakpoint = 0;
				for (int j = 0; j < words[i].length(); j++) { // Iterate over each character in the word, until:
					if (metrics.stringWidth(words[i].substring(breakpoint, j)) > width - start.x) { // The selected portion of the word is longer than the line.
						g.drawString(words[i].substring(breakpoint, j-1), start.x, start.y + advance); // Draw that part of the word, move the breakpoint,
						breakpoint = j-1;															   // and advance the line.
						advance += metrics.getHeight();
						if (advance > height - start.y) break;  // Again, stop drawing if we get to the end.
					}
				}
				buffer = words[i].substring(breakpoint) + " "; // Put whatever is left over back into the empty buffer.
			} else {
				buffer += words[i] + " "; // Normal case: Add the word to the buffer (either on a new line or still in progress).
			}
			if (advance > height - start.y) break; // Continuation of special case break out.
		}
		if (advance < height - start.y) { // If we still have space to draw:
			g.drawString(buffer, start.x, start.y + advance); // Draw the last piece of the buffer
		}
		return new Point(metrics.stringWidth(buffer) + start.x, advance + start.y); // The cursor goes at the end of the last line of text.
	}
	
	/**
	 * @return A new instance of this UI class.
	 */
	public static ComponentUI createUI() {
		return new BasicNotepageUI();
	}
	
	/*
	 * (non-Javadoc)
	 * @see main.NotepageUI#installUI(javax.swing.JComponent)
	 */
	public void installUI(JComponent c) {
		c.addMouseListener(this);
		c.addMouseMotionListener(this);
		c.addKeyListener(this);
	}
	
	/*
	 * (non-Javadoc)
	 * @see main.NotepageUI#uninstallUI(javax.swing.JComponent)
	 */
	public void uninstallUI(JComponent c) {
		c.removeMouseListener(this);
		c.removeMouseMotionListener(this);
		c.removeKeyListener(this);
	}

	/**
	 * Stop all editing and reflect changes to the model.
	 * 
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e) {
		Notepage p = (Notepage) e.getComponent();
		if (currentStroke != null) {
			if (mousePressed) {
				currentStroke.addPoint(e.getPoint());
			}
			p.addStroke(currentStroke);
		}
		if (currentText != null && currentText.size() > 0) {
			p.addText(currentText);
		}
		currentStroke = null;
		currentText = null;
		mousePressed = false;
	}

	/**
	 * Place a text insertion point (call repaint).
	 * 
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent e) {
		Notepage p = (Notepage) e.getComponent();
		if (currentText != null) {
			p.addText(currentText);
		}
		currentText = new NotepageText();
		currentText.setLocation(e.getPoint());
		currentText.setFont(p.getFont());
		currentText.setColor(p.getColor());
		p.repaint();
	}
	
	/**
	 * Begin drawing a stroke, grab focus for the component.
	 * 
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent e) {
		Notepage p = (Notepage) e.getComponent();
		p.requestFocus();
		mousePressed = true;
		currentStroke = new NotepageStroke();
		currentStroke.addPoint(e.getPoint());
		currentStroke.setPencolor(p.getColor());
		currentStroke.setPenwidth(p.getPenWidth());
	}

	/**
	 * Stop drawing a stroke, reflect changes to the model.
	 * 
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent e) {
		if (currentStroke != null) {
			if (mousePressed) {
				currentStroke.addPoint(e.getPoint());
			}
			Notepage p = (Notepage) e.getComponent();
			p.addStroke(currentStroke);
		}
		currentStroke = null;
		mousePressed = false;		
	}

	/**
	 * Add a typed character to the current text field (call repaint).
	 * 
	 * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
	 */
	public void keyTyped(KeyEvent e) {
		if (currentText != null) {
			if (e.getKeyChar() == KeyEvent.VK_BACK_SPACE) {
				currentText.backspace();
			} else {
				currentText.addText("" + e.getKeyChar());
			}
			e.getComponent().repaint();
		}
	}

	/**
	 * Continue drawing a stroke (call repaint).
	 * 
	 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
	 */
	public void mouseDragged(MouseEvent e) {
		if (mousePressed) {
			currentStroke.addPoint(e.getPoint());
			e.getComponent().repaint();
		}
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e) { } // Unused

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 */
	public void mouseMoved(MouseEvent e) { } // Unused

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
	 */
	public void keyPressed(KeyEvent e) { } // Unused

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
	 */
	public void keyReleased(KeyEvent e) { } // Unused
}
