package main.notepage;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.util.HashSet;
import java.util.Vector;

import javax.swing.JComponent;
//import javax.swing.UIManager; // UIManager hook isn't working right.  FIX THIS.
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


/**
 * Implementation class for the Notepage component.
 * 
 * @author Wervyn
 * @version 1.0
 */
@SuppressWarnings("serial")
public class Notepage extends JComponent implements ChangeListener {

	/**
	 * Constant representing Text Mode state.
	 */
	public static final int TEXT_MODE = 0;
	
	/**
	 * Constant representing Pen Mode state.
	 */
	public static final int PEN_MODE = 1;
	
	/** The current model for this Notepage. */
	private NotepageModel model;
	
	/** The current drawing mode of this Notepage. */
	private int mode;
	
	/** The drawing color of this Notepage. */
	private Color color;
	
	/** The drawing font of this Notepage. */
	private Font font;
	
	/** The pen size of this Notepage. */
	private int penWidth;

	/**
	 * Default constructor for a new Notepage.
	 */
	public Notepage() {
		setModel(new NotepageModel());
		setMode(PEN_MODE);
		setColor(Color.black);
		setFont(new Font("Arial", Font.BOLD, 14));
		setPenWidth(3);
		updateUI();
	}
	
	/**
	 * Add a tag to this Notepage.
	 * 
	 * @param tag The tag to add.
	 */
	public void addTag(String tag) {
		model.addTag(tag);
	}
	
	/**
	 * Remove a tag from this Notepage.
	 * 
	 * @param tag The tag to remove.
	 */
	public void removeTag(String tag) {
		model.removeTag(tag);
	}
	
	/**
	 * Get all tags associated with this Notepage.
	 * 
	 * @return A list of tags.
	 */
	public HashSet<String> getTags() {
		return model.getTags();
	}
	
	/*
	 * (non-Javadoc)
	 * @see javax.swing.JComponent#updateUI()
	 */
	public void updateUI() {
//		setUI((NotepageUI) UIManager.getUI(this)); // Couldn't get this to work for some reason, not sure what the problem is.
		setUI((NotepageUI) BasicNotepageUI.createUI()); // Shortcutting around it for now.
		invalidate();
	}
	
	/*
	 * (non-Javadoc)
	 * @see javax.swing.JComponent#getUIClassID()
	 */
	public String getUIClassID() {
		return NotepageUI.UI_CLASS_ID;
	}
	
	/**
	 * Repaint this component.
	 * 
	 * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
	 */
	public void stateChanged(ChangeEvent arg0) {
		repaint();
	}
	
	/*
	 * (non-Javadoc)
	 * @see javax.swing.JComponent#getPreferredSize()
	 */
	public Dimension getPreferredSize() {
		return new Dimension(640,480);
	}
	
	/*
	 * (non-Javadoc)
	 * @see javax.swing.JComponent#getMinimumSize()
	 */
	public Dimension getMinimumSize() {
		return new Dimension(320,240);
	}
	
	/**
	 * Add a stroke to this Notepage.
	 * 
	 * @param stroke The stroke to add.
	 */
	public void addStroke(NotepageStroke stroke) {
		if (mode == PEN_MODE) { // Protection, avoids UI noise.
			model.addObject(stroke);
		}
	}
	
	/**
	 * Add a text field to this Notepage.
	 * 
	 * @param text The text to add.
	 */
	public void addText(NotepageText text) {
		if (mode == TEXT_MODE) { // Protection, avoids UI noise.
			model.addObject(text);
		}
	}

	/**
	 * Get a list of objects drawn on this Notepage.
	 * 
	 * @return A list of drawn objects.
	 */
	public Vector<NotepagePrimative> getObjects() {
		return model.getObjects();
	}

	/**
	 * Set a new model for this Notepage.
	 * 
	 * @param m The new model.
	 */
	public void setModel(NotepageModel m) {
		if (model != null)
			model.removeChangeListener(this);
		model = m;
		model.addChangeListener(this);
	}
	
	/**
	 * Get the model for this Notepage.
	 * 
	 * @return The model for the Notepage.
	 */
	public NotepageModel getModel() {
		return model;
	}
	
	/**
	 * Set the UI for this Notepage.
	 * 
	 * @param ui The new UI.
	 */
	public void setUI(NotepageUI ui) {
		super.setUI(ui);
	}

	/**
	 * Modifier for the drawing mode.
	 * 
	 * @param mode The new mode for the Notepage.
	 */
	public void setMode(int mode) {
		this.mode = mode;
		repaint();
	}

	/**
	 * Accessor for the drawing mode.
	 * 
	 * @return The mode of the Notepage.
	 */
	public int getMode() {
		return mode;
	}
	
	/**
	 * Modifier for the drawing color.
	 * 
	 * @param color The new color for the Notepage.
	 */
	public void setColor(Color color) {
		this.color = color;
	}

	/**
	 * Accessor for the drawing color.
	 * 
	 * @return The color of the Notepage.
	 */
	public Color getColor() {
		return color;
	}
	
	/**
	 * Modifier for the drawing font.
	 * 
	 * @param font The new font for the Notepage.
	 */
	public void setFont(Font font) {
		this.font = font;
	}
	
	/**
	 * Accessor for the drawing font.
	 * 
	 * @param font The font of the Notepage.
	 */
	public Font getFont() {
		return font;
	}
	
	/**
	 * Modifier for the pen width.
	 * 
	 * @param penWidth The new pen width for the Notepage.
	 */
	public void setPenWidth(int penWidth) {
		this.penWidth = penWidth;
	}
	
	/**
	 * Accessor for the pen width.
	 * 
	 * @return The pen width of the Notepage.
	 */
	public int getPenWidth() {
		return penWidth;
	}
}
