package main.notepage;

import java.util.Iterator;
import java.util.HashSet;
import java.util.Vector;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * A basic data model class for a Notepage.
 * 
 * @author Wervyn
 * @version 1.0
 */
public class NotepageModel {

	/** A list of drawn objects associated with this Notepage. */
	private Vector<NotepagePrimative> objects;
	
	/** A list of listeners to inform of changes to this model's data. */
	private Vector<ChangeListener> listeners;
	
	/** A list of tags associated with this Notepage. */
	private HashSet<String> tags;
	
	/**
	 * Default constructor, initializes empty data structures.
	 */
	public NotepageModel() {
		listeners = new Vector<ChangeListener>();
		tags = new HashSet<String>();
		setObjects(new Vector<NotepagePrimative>());
	}

	/**
	 * Set the object list to a predefined set, discarding current data.
	 * 
	 * @param objects A new list of drawn objects.
	 */
	public void setObjects(Vector<NotepagePrimative> objects) {
		this.objects = objects;
		this.fireChangeEvent();
	}

	/**
	 * @return The objects drawn on this Notepage.
	 */
	public Vector<NotepagePrimative> getObjects() {
		return objects;
	}
	
	/**
	 * Add a new object to the list of drawn objects.
	 * 
	 * @param object The object to be added.
	 */
	public void addObject(NotepagePrimative object) {
		this.objects.addElement(object);
		this.fireChangeEvent();
	}
	
	/**
	 * Remove an existing object from the list of drawn objects.
	 * 
	 * @param object The object to be removed.
	 */
	public void removeObject(NotepagePrimative object) {
		this.objects.removeElement(object);
		this.fireChangeEvent();
	}
	
	/**
	 * Inform all ChangeListeners of a change to this model's data.
	 */
	public void fireChangeEvent() {
		Iterator<ChangeListener> i = listeners.iterator();
		while (i.hasNext()) {
			ChangeListener l = i.next();
			l.stateChanged(new ChangeEvent(this));
		}
	}
	
	/**
	 * Add a ChangeListener to this model.
	 * 
	 * @param l The ChangeListener to add.
	 */
	public void addChangeListener(ChangeListener l) {
		this.listeners.addElement(l);
	}
	
	/**
	 * Remove a ChangeListener from this model.
	 * 
	 * @param l The ChangeListener to remove.
	 */
	public void removeChangeListener(ChangeListener l) {
		this.listeners.removeElement(l);
	}

	/**
	 * Add a tag to this model's data.
	 * 
	 * @param tag The tag to add.
	 */
	public void addTag(String tag) {
		tags.add(tag);
	}
	
	/**
	 * Remove a tag from this model's data.
	 * 
	 * @param tag The tag to remove.
	 */
	public void removeTag(String tag) {
		tags.remove(tag);
	}
	
	/**
	 * Get all tags associated with this model.
	 * 
	 * @return The list of tags.
	 */
	public HashSet<String> getTags() {
		return tags;
	}
}
